//------------------------------------------------------------------------------
//
// Breakout.  Written by Punk Jozsef.
//
// Copyright 2007-2008 Intuitive Computers.  All rights reserved.
//
// Breakout sample for Intuitive Media Studio.
//
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Constants
//------------------------------------------------------------------------------

// definition of game states
enum State {
	SMenu,
	SWait,
	SPlay,
	SOut
};

//------------------------------------------------------------------------------
// Global Variables
//------------------------------------------------------------------------------

State		gState;

int			gLevel, gScore, gBalls;
int			gX, gY, gDX, gDY, gSX;

int			gSprite [6] = { 0, Brick1, Brick2, Brick3, Brick4, Brick5 };
int			gNum [10] = { Num0, Num1, Num2, Num3, Num4, Num5, Num6, Num7, Num8, Num9 };
int			gBrick [5][15];

//------------------------------------------------------------------------------
// Render function
//------------------------------------------------------------------------------

void Render ()

{
	int	i, j, x, score, s [6];

	FrameStart ();
	FrameClear ();

	SpriteDraw (0, 0, Background);

	if (gState == SMenu) {
		SpriteDraw (30, 30, Breakout);
		SpriteDraw (30, 200, Start);
		SpriteDraw (30, 230, Quit);
	} else {
		SpriteDraw (0, 256, Score);

		score = gScore;
		for (i = 0; score > 0; i += 1) {
			s [i] = gNum [score - (score / 10) * 10];
			score /= 10;
		}

		if (i == 0) {
			i = 1;
			s [0] = gNum [0];
		}

		x = 72;
		for (j = i - 1; j >= 0; j -= 1) {
			SpriteDraw (x, 256, s [j]);
			x += 10;
		}

		SpriteDraw (400, 256, Level);
		SpriteDraw (466, 256, gNum [gLevel]);

		x = 240 - gBalls * 8;
		for (i = 0; i < gBalls; i += 1) {
			SpriteDraw (x, 256, Ball);
			x += 16;
		}

		for (j = 0; j < 5; j += 1) {
			for (i = 0; i < 15; i += 1) {
				if (gBrick [j][i] != 0)
					SpriteDraw (i * 32, j * 16, gSprite [gBrick [j][i]]);
			}
		}

		if (gBalls > 0)
			SpriteDraw (gX, gY, Ball);

		SpriteDraw (gSX, 240, Stick);

		if (gState == SWait) {
			if (gBalls > 0)
				SpriteDraw (176, 128, PressX);
			else
				SpriteDraw (176, 128, GameOver);
		}
	}

	FrameEnd ();

	return;
}

//------------------------------------------------------------------------------
// Update function
//------------------------------------------------------------------------------

void Update ()

{
	int		bx, by, dx, out;
	
	if (gState == SMenu) {
		if (ControlValue (CPad0, CPadStart) > 0.0) {
			gLevel = 1;
			gBalls = 3;
			gScore = 0;
			gX = 100;
			gY = 200;
			gDX = 2;
			gDY = -3;
			gSX = 240 - 32;
			gState = SWait;
		}
		return;
	}

	if (ControlValue (CPad0, CPadRight) > 0.0) {
		if (gSX <= 480 - 64 - 8)
			gSX += 8;
	}
	if (ControlValue (CPad0, CPadLeft) > 0.0) {
		if (gSX >= 8)
			gSX -= 8;
	}

	if (gState == SWait) {
		gX = gSX + 32 - 8;
		gY = 240 - 16;
		if (ControlValue (CPad0, CPadX) > 0.0) {
			if (gBalls > 0)
				gState = SPlay;
			else
				gState = SMenu;
		}
		return;
	}

	gX += gDX;
	gY += gDY;

	if (gState == SOut) {
		if (gY >= 272) {
			gBalls -= 1;
			gDY = -gDY;
			SoundPlay (Sound3, 1.0, 0.0);
			gState = SWait;
		}
		return;
	}

	if (gX < 0) {
		gX = 0;
		gDX = -gDX;
	}
	if (gX > 480 - 16) {
		gX = 480 - 16;
		gDX = -gDX;
	}

	if (gY < 0) {
		gY = 0;
		gDY = -gDY;
	}
	if (gY > 240 - 16) {
		dx = gX + 8 - (gSX + 32);
		if (dx < 0)
			dx = -dx;
		if (dx <= 32) {
			gY = 240 - 16;
			if (gDX < 0)
				gDX = -dx / 8 - 1;
			else
				gDX = dx / 8 + 1;
			gDY = -gDY;
			SoundPlay (Sound2, 1.0, 1.0);
		} else {
			gState = SOut;
		}
	}

	if (gY < 16 * 5) {
		bx = (gX + 8) / 32;
		by = gY / 16;
		if (gBrick [by][bx] != 0) {
			gBrick [by][bx] = 0;
			gDY = -gDY;
			SoundPlay (Sound1, 1.0, -1.0);
			gScore += 1;
		}
	}

	return;
}

//------------------------------------------------------------------------------
// main function
//------------------------------------------------------------------------------

void main ()

{
	int		i, j;

	DisplaySize (480, 272);

	for (i = 0; i < 5; i += 1) {
		for (j = 0; j < 15; j += 1) {
			gBrick [i][j] = i + 1;
		}
	}

	gState = SMenu;

	while (ControlValue (CPad0, CPadSelect) == 0.0) {
		Update ();
		Render ();
		System ();
	}

	return;
}

